/**
 * 
 */
package org.gcube.portlets.user.td.csvimportwidget.client;

import org.gcube.portlets.user.td.csvimportwidget.client.general.WizardCard;
import org.gcube.portlets.user.td.csvimportwidget.client.progress.CSVImportProgressBarUpdater;
import org.gcube.portlets.user.td.csvimportwidget.client.progress.CSVImportProgressListener;
import org.gcube.portlets.user.td.csvimportwidget.client.progress.CSVImportProgressUpdater;
import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
import org.gcube.portlets.user.td.gwtservice.shared.TRId;
import org.gcube.portlets.user.td.gwtservice.shared.csv.CSVImportSession;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FlexTable;
import com.sencha.gxt.core.client.util.Margins;
import com.sencha.gxt.widget.core.client.FramedPanel;
import com.sencha.gxt.widget.core.client.ProgressBar;
import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutData;
import com.sencha.gxt.widget.core.client.container.VBoxLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VBoxLayoutContainer.VBoxLayoutAlign;

//import com.allen_sauer.gwt.log.client.Log;
//import com.google.gwt.user.client.Command;

/**
 * 
 * @author "Giancarlo Panichi" <a
 *         href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a>
 * 
 */
public class CSVOperationInProgressCard extends WizardCard implements
CSVImportProgressListener {

	public static final int STATUS_POLLING_DELAY = 1000;
	protected CSVOperationInProgressCard thisCard;
	protected CSVImportSession importSession;
	protected CSVImportProgressUpdater progressUpdater;

	public CSVOperationInProgressCard(final CSVImportSession importSession) {
		super("Operation In Progress", "");

		this.importSession = importSession;
		thisCard = this;

		VBoxLayoutContainer operationInProgressPanel = new VBoxLayoutContainer();
		operationInProgressPanel.setVBoxLayoutAlign(VBoxLayoutAlign.CENTER);

		final FlexTable description = new FlexTable();
		//FlexCellFormatter cellFormatter = description.getFlexCellFormatter();
		description.setCellSpacing(10);
		description.setCellPadding(4);
		description.setBorderWidth(0);

		// display:block;vertical-align:text-top;
		description.setHTML(0, 0,
				"<span style=\"font-weight:bold;\";>Document: </span>");
		description.setText(0, 1, "CSV File");
		description.setHTML(1, 0,
				"<span style=\"font-weight:bold;\";>Source: </span>");
		description.setText(1, 1, importSession.getSource().getName());

		description.setHTML(2, 0,
				"<span style=\"font-weight:bold;\";>File: </span>");
		description.setText(2, 1, importSession.getTabResource().getName());

		
		FramedPanel summary = new FramedPanel();
		summary.setHeadingText("Import Summary");
		summary.setWidth(400);
		summary.add(description);
		operationInProgressPanel.add(summary, new BoxLayoutData(new Margins(20,
				5, 10, 5)));

		ProgressBar progressBar = new ProgressBar();
		operationInProgressPanel.add(progressBar, new BoxLayoutData(
				new Margins(10, 5, 10, 5)));

		progressUpdater = new CSVImportProgressUpdater();
		progressUpdater.addListener(new CSVImportProgressBarUpdater(progressBar));

		progressUpdater.addListener(this);

		setContent(operationInProgressPanel);

	}
	//columnToImportMask

	public void importCSV() {
		TDGWTServiceAsync.INSTANCE.startCSVImport(importSession, new AsyncCallback<Void>() {

					@Override
					public void onSuccess(Void result) {
						progressUpdater.scheduleRepeating(STATUS_POLLING_DELAY);
					}

					@Override
					public void onFailure(Throwable caught) {
						showErrorAndHide("Error in importCSV",
								"An error occured in importCSV", "", caught);
					}
				});
	}

	@Override
	public void setup() {
		getWizardWindow().setEnableBackButton(false);
		setBackButtonVisible(false);
		setNextButtonVisible(false);
		getWizardWindow().setEnableNextButton(false);
		getWizardWindow().setNextButtonToFinish();
		importCSV();
	}

	@Override
	public void operationInitializing() {

	}

	@Override
	public void operationUpdate(float elaborated) {

	}

	@Override
	public void operationComplete(final TRId trId) {
		// final String tableId,final String tableResourceId) {

		Command sayComplete = new Command() {
			public void execute() {
				try {
					getWizardWindow().close(false);
					Log.info("fire Complete: tabular resource " + trId.getId());
					Log.info("fire Complete: tableId " + trId.getTableId());

					getWizardWindow().fireCompleted(trId);

				} catch (Exception e) {
					Log.error("fire Complete :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setNextButtonCommand(sayComplete);

		setNextButtonVisible(true);
		getWizardWindow().setEnableNextButton(true);
	}

	@Override
	public void operationFailed(Throwable caught, String reason,
			String failureDetails) {

	}

}
