/**
 * 
 */
package org.gcube.portlets.user.td.csvimportwidget.client;

import org.gcube.portlets.user.td.csvimportwidget.client.general.WizardCard;
import org.gcube.portlets.user.td.gwtservice.shared.TabResource;
import org.gcube.portlets.user.td.gwtservice.shared.csv.CSVImportSession;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.sencha.gxt.widget.core.client.FramedPanel;
import com.sencha.gxt.widget.core.client.box.AlertMessageBox;
import com.sencha.gxt.widget.core.client.button.TextButton;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;
import com.sencha.gxt.widget.core.client.form.FieldLabel;
import com.sencha.gxt.widget.core.client.form.FieldSet;
import com.sencha.gxt.widget.core.client.form.TextArea;
import com.sencha.gxt.widget.core.client.form.TextField;

/**
 * 
 * @author "Giancarlo Panichi" <a
 *         href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a>
 * 
 */
public class CSVTableDetailCard extends WizardCard {

	protected final String TABLEDETAILPANELWIDTH = "100%";
	protected final String TABLEDETAILPANELHEIGHT = "100%";
	protected final String FORMWIDTH = "538px";

	protected CSVImportSession importSession;
	protected CSVTableDetailCard thisCard;

	protected VerticalLayoutContainer p;
	protected VerticalPanel tableDetailPanel;

	protected TextField name;
	protected TextArea description;
	protected TextArea rights;

	protected TextButton checkButton;

	// protected ComboBox<Agencies> combo=null;

	TabResource detail = new TabResource();

	public CSVTableDetailCard(final CSVImportSession importSession) {
		super("Tabular Resource Detail", "");

		this.importSession = importSession;
		thisCard = this;

		tableDetailPanel = new VerticalPanel();

		tableDetailPanel.setSpacing(4);
		tableDetailPanel.setWidth(TABLEDETAILPANELWIDTH);
		tableDetailPanel.setHeight(TABLEDETAILPANELHEIGHT);

		FramedPanel form = new FramedPanel();
		form.setHeadingText("Details");
		form.setWidth(FORMWIDTH);

		FieldSet fieldSet = new FieldSet();
		fieldSet.setHeadingText("Information");
		fieldSet.setCollapsible(false);
		form.add(fieldSet);

		p = new VerticalLayoutContainer();
		fieldSet.add(p);

		name = new TextField();
		name.setAllowBlank(false);
		name.setEmptyText("Enter a name...");
		name.setValue(importSession.getLocalFileName());
		name.setAllowBlank(false);
		p.add(new FieldLabel(name, "Name"), new VerticalLayoutData(1, -1));

		description = new TextArea();
		description.setAllowBlank(false);
		description.setEmptyText("Enter a description...");
		description.setValue("File CSV");
		description.setAllowBlank(false);
		p.add(new FieldLabel(description, "Description"),
				new VerticalLayoutData(1, -1));

		rights = new TextArea();
		rights.setEmptyText("Enter rights...");
		rights.setAllowBlank(false);
		p.add(new FieldLabel(rights, "Rights"), new VerticalLayoutData(1, -1));

		checkButton = new TextButton("Save");
		checkButton.addSelectHandler(new SelectHandler() {

			@Override
			public void onSelect(SelectEvent event) {
				checkData();
			}
		});
		form.addButton(checkButton);
		tableDetailPanel.add(form);

		setContent(tableDetailPanel);

	}

	protected void checkData() {
		if (name.getValue() == null || name.getValue().isEmpty()||!name.isValid()
				|| description.getValue() == null
				|| description.getValue().isEmpty()||!description.isValid()
				|| rights.getValue() == null || rights.getValue().isEmpty()||!rights.isValid()

		) {
			AlertMessageBox d = new AlertMessageBox("Attention!",
					"Fill in all fields");
			d.addHideHandler(new HideHandler() {

				@Override
				public void onHide(HideEvent event) {
					//
				}

			});
			d.show();
			return;
		} else {
			name.setReadOnly(true);
			description.setReadOnly(true);
			rights.setReadOnly(true);
			checkButton.disable();
			getWizardWindow().setEnableNextButton(true);
		}
	}

	@Override
	public void setup() {
		Command sayNextCard = new Command() {

			@Override
			public void execute() {
				detail.setName(name.getCurrentValue());
				detail.setDescription(description.getCurrentValue());
				detail.setRight(rights.getCurrentValue());

				importSession.setTabResource(detail);

				CSVOperationInProgressCard csvOperationInProgressCard = new CSVOperationInProgressCard(
						importSession);
				getWizardWindow().addCard(csvOperationInProgressCard);
				Log.info("NextCard CSVOperationInProgressCard");
				getWizardWindow().nextCard();

			}

		};

		getWizardWindow().setNextButtonCommand(sayNextCard);

		Command sayPreviousCard = new Command() {
			public void execute() {
				try {
					getWizardWindow().previousCard();
					getWizardWindow().removeCard(thisCard);
					Log.info("Remove SDMXTableDetailCard");
				} catch (Exception e) {
					Log.error("sayPreviousCard :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setPreviousButtonCommand(sayPreviousCard);
		getWizardWindow().setEnableNextButton(false);

	}

}
